process SHAPEIT5_PHASECOMMON {
    tag "$meta.id"
    label 'process_medium'

    conda "${moduleDir}/environment.yml"
    container "${ workflow.containerEngine == 'singularity' && !task.ext.singularity_pull_docker_container ?
        'https://depot.galaxyproject.org/singularity/shapeit5:1.0.0--h0c8ee15_0':
        'biocontainers/shapeit5:1.0.0--h0c8ee15_0'}"

    input:
        tuple val(meta) , path(input), path(input_index), path(pedigree), val(region), path(map)
        tuple val(meta2), path(reference), path(reference_index)
        tuple val(meta3), path(scaffold), path(scaffold_index)

    output:
        tuple val(meta), path("*.{vcf,bcf,vcf.gz,bcf.gz}"), emit: phased_variant
        path "versions.yml"                               , emit: versions

    when:
        task.ext.when == null || task.ext.when

    script:
    def args   = task.ext.args   ?: ''
    def prefix = task.ext.prefix ?: "${meta.id}"
    def suffix = task.ext.suffix ?: "vcf.gz"

    if ("$input" == "${prefix}.${suffix}") error "Input and output names are the same, set prefix in module configuration to disambiguate!"

    def map_command       = map       ? "--map $map"             : ""
    def reference_command = reference ? "--reference $reference" : ""
    def scaffold_command  = scaffold  ? "--scaffold $scaffold"   : ""
    def pedigree_command  = pedigree  ? "--pedigree $pedigree"   : ""

    """
    SHAPEIT5_phase_common \\
        $args \\
        --input $input \\
        $map_command \\
        $reference_command \\
        $scaffold_command \\
        $pedigree_command \\
        --region $region \\
        --thread $task.cpus \\
        --output ${prefix}.${suffix}

    cat <<-END_VERSIONS > versions.yml
    "${task.process}":
        shapeit5: "\$(SHAPEIT5_phase_common | sed -nr '/Version/p' | grep -o -E '([0-9]+.){1,2}[0-9]' | head -1)"
    END_VERSIONS
    """

    stub:
    def args   = task.ext.args   ?: ''
    def prefix = task.ext.prefix ?: "${meta.id}"
    def suffix = task.ext.suffix ?: "vcf.gz"
    """
    touch ${prefix}.${suffix}

    cat <<-END_VERSIONS > versions.yml
    "${task.process}":
        shapeit5: "\$(SHAPEIT5_phase_common | sed -nr '/Version/p' | grep -o -E '([0-9]+.){1,2}[0-9]' | head -1)"
    END_VERSIONS
    """
}
